==> eyflutter_uikit-0.0.5%2B9.tar.gz: lib/uikit/state/base_list_state.dart <==
import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_uikit/uikit/bars/action_bar.dart';
import 'package:eyflutter_uikit/uikit/images/asset_image_provider.dart';
import 'package:eyflutter_uikit/uikit/refresh/cus_refresh_controller.dart';
import 'package:eyflutter_uikit/uikit/refresh/refresh_utils.dart';
import 'package:eyflutter_uikit/uikit/refresh/refresh_widget.dart';
import 'package:eyflutter_uikit/uikit/state/base_state.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:provider/provider.dart';

abstract class BaseListModel<ITEM> extends ProviderListModel<ITEM> {
  int _page = 1;
  bool isNoMore = false;
  bool isEmptyData = false;
  Map<String, dynamic> _arguments = {};

  //刷新组件controller
  CusRefreshController? refreshController;

  /// get model listener
  OnBaseModelListener get modelListener;

  /// 获取当前分页索引
  int get pageIndex => _page;

  Map<String, dynamic> get arguments => _arguments;

  /// state页面初始化
  void onInitState() {
    _page = 1;
    refreshController?.isAutoTriggerLoad = true;
  }

  void onPreCreate(BuildContext context, Map<String, dynamic> arguments) {}

  /// 设置空视图隐藏与显示
  void showEmptyView(bool show) {
    isEmptyData = show;
    notifierLazy();
  }

  /// 分页接口请求完成后执行，配置相关状态
  /// [response] 分页数据对象,请求失败时不需要传入
  void finishRequest({Map<String, dynamic>? response}) {
    var firstFieldName = DioManager.instance.configInterceptor?.isFirstPageKey() ?? "";
    var isFirstPage = response?[firstFieldName] ?? false;
    if ((isFirstPage is! bool) || isFirstPage) {
      super.clear();
    } else {
      _page++;
    }
    var nextFieldName = DioManager.instance.configInterceptor?.hasNextPageKey() ?? "";
    var hasNextPage = response?[nextFieldName] ?? false;
    if ((hasNextPage is! bool) || !hasNextPage) {
      isNoMore = true;
    } else {
      isNoMore = false;
    }
    var countFieldName = DioManager.instance.configInterceptor?.countKey() ?? "";
    var count = response?[countFieldName] ?? 0;
    if (_page == 1 && isNoMore && count == 0) {
      isEmptyData = true;
    } else {
      isEmptyData = false;
    }
  }

  /// 是否首页数据
  bool isFirstPage(Map<String, dynamic> response) {
    if (response.isEmptyMap()) {
      return true;
    }
    var fieldName = DioManager.instance.configInterceptor?.isFirstPageKey() ?? "";
    var isFirstPage = response[fieldName];
    if (isFirstPage is! bool) {
      return true;
    }
    return isFirstPage;
  }

  void finishRefresh() {
    refreshController?.finishRefresh(success: true, noMore: isNoMore);
  }

  void finishLoad() {
    Future.delayed(Duration(milliseconds: 200), () => refreshController?.finishLoad(success: true, noMore: isNoMore));
  }

  /// 刷新数据
  void refresh(BuildContext context) {
    //网络异常时显示失败视图并结束请求
    NetworkManager.instance.isConnected().then((netStatus) {
      if (netStatus ?? true) {
        _page = 1;
        isNoMore = false;
        refreshController?.isAutoTriggerLoad = true;
        modelListener.onRequestData(context, RequestWay.refresh);
      } else {
        finishRefresh();
      }
    });
  }

  /// 加载数据
  void load(BuildContext context) {
    //网络异常时显示失败视图并结束请求
    NetworkManager.instance.isConnected().then((netStatus) {
      if (netStatus ?? true) {
        modelListener.onRequestData(context, RequestWay.load);
      } else {
        finishLoad();
      }
    });
  }
}

abstract class BaseListState<T extends StatefulWidget, M extends BaseListModel> extends BaseState<T> {
  bool _isEnableRefresh = true;
  bool _isEnableLoad = true;
  late M _model;

  /// 创建notify model
  M get createModel;

  /// 获取模型对象
  M get model => _model;

  /// 页面背景颜色
  Color get backgroundColor => Color(0xFFF4F4F4);

  /// 背景图片地址
  String backgroundImage() {
    return "";
  }

  /// 背景对齐方式
  AlignmentGeometry backgroundAlignment() {
    return Alignment.bottomCenter;
  }

  /// 是否启动刷新
  /// 注：该方法需要在initState中调用
  void enableRefresh(bool isEnable) {
    this._isEnableRefresh = isEnable;
  }

  /// 是否启用加载
  /// 注：该方法需要在initState中调用
  void enableLoad(bool isEnable) {
    this._isEnableLoad = isEnable;
  }

  @override
  void onPreCreate(BuildContext context, String routeName, Map<String, dynamic> arguments) {
    _model = createModel;
    _model._arguments = arguments;
    _model.onPreCreate(context, arguments);
  }

  @override
  void dispatchLangState(Locale locale) {
    super.dispatchLangState(locale);
    model.refreshController?.isAutoTriggerLoad = true;
    model.modelListener.onRequestData(context, RequestWay.refresh);
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    return overrideLocalizations(
      context: context,
      child: ChangeNotifierProvider<M>(
        create: (_) => model,
        child: Consumer<M>(
          builder: (context, model, _) {
            return Scaffold(
              appBar: onActionBar(),
              body: Container(
                color: backgroundColor,
                alignment: model.isEmptyData ? Alignment.center : null,
                child: _contentWidget(context),
              ),
            );
          },
        ),
      ),
    );
  }

  Widget _contentWidget(BuildContext context) {
    if (this.backgroundImage().isEmptyString) {
      return Column(
        children: [onBuildHeadView(), Expanded(child: _buildRefreshWidget(context)), onBuildBottomView(context)],
      );
    } else {
      return Stack(
        children: [
          Container(
            decoration: BoxDecoration(
                image: DecorationImage(
              image:
                  AssetImageProvider.render(this.backgroundImage(), isProjectImage: !PluginUtils.instance.isRunPlugin),
              alignment: this.backgroundAlignment(),
              fit: BoxFit.fitWidth,
            )),
          ),
          Column(
            children: [onBuildHeadView(), Expanded(child: _buildRefreshWidget(context)), onBuildBottomView(context)],
          )
        ],
      );
    }
  }

  /// 构建顶部导航条
  ActionBar? onActionBar() {
    return null;
  }

  /// 构建页面顶部视图
  Widget onBuildHeadView() {
    return Container();
  }

  /// 空视图对象
  Widget onEmptyRenderObject() {
    return Container();
  }

  /// 渲染视图top对象
  void onRenderTopObject(List<Widget> list) {}

  /// 构建渲染对象
  void onRenderObject(List<Widget> list) {}

  /// 渲染视图bottom对象
  Widget onBuildBottomView(BuildContext context) {
    return Container();
  }

  Widget _buildRefreshWidget(BuildContext context) {
    return RefreshWidget(
        listStyle: RefreshListStyle.custom,
        scrollMaxRatioLoad: 1,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: _bindListWidgets(),
        ),
        loadIndicatorWidget: Container(
          width: 86,
          alignment: Alignment.center,
          child: model.isNoMore
              ? Text(
                  RefreshUtils.instance.refreshConfig?.noMoreText() ?? "",
                  style: TextStyle(color: Color(0xb3333333), fontSize: 12),
                )
              : RefreshUtils.instance.refreshConfig?.loadingWidget(),
        ),
        enableRefresh: _isEnableRefresh,
        enableLoad: _isEnableLoad,
        initCall: (controller) {
          model.refreshController = controller;
        },
        onRefresh: (EasyRefreshController controller) async {
          model.refresh(context);
        },
        onLoad: (EasyRefreshController controller) {
          model.load(context);
        });
  }

  List<Widget> _bindListWidgets() {
    List<Widget> lst = [];
    onRenderTopObject(lst);
    if (model.isEmptyData) {
      lst.add(onEmptyRenderObject());
    } else {
      onRenderObject(lst);
    }
    return lst;
  }
}

==> eyflutter_uikit-0.0.6%2B9.tar.gz: lib/uikit/state/base_list_state.dart <==
import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_uikit/uikit/bars/action_bar.dart';
import 'package:eyflutter_uikit/uikit/images/asset_image_provider.dart';
import 'package:eyflutter_uikit/uikit/refresh/cus_refresh_controller.dart';
import 'package:eyflutter_uikit/uikit/refresh/refresh_utils.dart';
import 'package:eyflutter_uikit/uikit/refresh/refresh_widget.dart';
import 'package:eyflutter_uikit/uikit/state/base_state.dart';
import 'package:eyflutter_uikit/uikit/state/state_utils.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:provider/provider.dart';

abstract class BaseListModel<ITEM> extends ProviderListModel<ITEM> {
  int _page = 1;
  bool isNoMore = false;
  bool isEmptyData = false;
  Map<String, dynamic> _arguments = {};

  //刷新组件controller
  CusRefreshController? refreshController;

  /// get model listener
  OnBaseModelListener get modelListener;

  /// 获取当前分页索引
  int get pageIndex => _page;

  Map<String, dynamic> get arguments => _arguments;

  /// 重置页面属性
  void reset() {
    _page = 1;
    refreshController?.isAutoTriggerLoad = true;
  }

  void onPreCreate(BuildContext context, Map<String, dynamic> arguments) {}

  /// 设置空视图隐藏与显示
  void showEmptyView(bool show) {
    isEmptyData = show;
    notifierLazy();
  }

  /// 分页接口请求完成后执行，配置相关状态
  /// [response] 分页数据对象,请求失败时不需要传入
  void finishRequest({Map<String, dynamic>? response}) {
    var firstFieldName = DioManager.instance.configInterceptor?.isFirstPageKey() ?? "";
    var isFirstPage = response?[firstFieldName] ?? false;
    if ((isFirstPage is! bool) || isFirstPage) {
      super.clear();
    } else {
      _page++;
    }
    var nextFieldName = DioManager.instance.configInterceptor?.hasNextPageKey() ?? "";
    var hasNextPage = response?[nextFieldName] ?? false;
    if ((hasNextPage is! bool) || !hasNextPage) {
      isNoMore = true;
    } else {
      isNoMore = false;
    }
    var countFieldName = DioManager.instance.configInterceptor?.countKey() ?? "";
    var count = response?[countFieldName] ?? 0;
    if (_page == 1 && isNoMore && count == 0) {
      isEmptyData = true;
    } else {
      isEmptyData = false;
    }
  }

  /// 是否首页数据
  bool isFirstPage(Map<String, dynamic> response) {
    return StateUtils.instance.isFirstPage(response);
  }

  void finishRefresh() {
    refreshController?.finishRefresh(success: true, noMore: isNoMore);
  }

  void finishLoad() {
    Future.delayed(Duration(milliseconds: 200), () => refreshController?.finishLoad(success: true, noMore: isNoMore));
  }

  /// 刷新数据
  void refresh(BuildContext context) {
    //网络异常时显示失败视图并结束请求
    NetworkManager.instance.isConnected().then((netStatus) {
      if (netStatus ?? true) {
        _page = 1;
        isNoMore = false;
        refreshController?.isAutoTriggerLoad = true;
        modelListener.onRequestData(context, RequestWay.refresh);
      } else {
        finishRefresh();
      }
    });
  }

  /// 加载数据
  void load(BuildContext context) {
    //网络异常时显示失败视图并结束请求
    NetworkManager.instance.isConnected().then((netStatus) {
      if (netStatus ?? true) {
        modelListener.onRequestData(context, RequestWay.load);
      } else {
        finishLoad();
      }
    });
  }
}

abstract class BaseListState<T extends StatefulWidget, M extends BaseListModel> extends BaseState<T> {
  bool _isEnableRefresh = true;
  bool _isEnableLoad = true;
  late M _model;

  /// 创建notify model
  M get createModel;

  /// 获取模型对象
  M get model => _model;

  /// 页面背景颜色
  Color get backgroundColor => Color(0xFFF4F4F4);

  /// 背景图片地址
  String get backgroundImage => "";

  /// 背景对齐方式
  AlignmentGeometry get backgroundAlignment => Alignment.bottomCenter;

  /// 是否启动刷新
  /// 注：该方法需要在initState中调用
  bool get enableRefresh => true;

  /// 是否启用加载
  /// 注：该方法需要在initState中调用
  bool get enableLoad => true;

  @override
  void onPreCreate(BuildContext context, String routeName, Map<String, dynamic> arguments) {
    _model = createModel;
    _model._arguments = arguments;
    _model.onPreCreate(context, arguments);
  }

  @override
  void dispatchLangState(Locale locale) {
    super.dispatchLangState(locale);
    model.refreshController?.isAutoTriggerLoad = true;
    model.modelListener.onRequestData(context, RequestWay.refresh);
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    return overrideLocalizations(
      context: context,
      child: ChangeNotifierProvider<M>(
        create: (_) => model,
        child: Consumer<M>(
          builder: (context, model, _) {
            return Scaffold(
              appBar: onActionBar(),
              body: Container(
                color: backgroundColor,
                alignment: model.isEmptyData ? Alignment.center : null,
                child: _contentWidget(context),
              ),
            );
          },
        ),
      ),
    );
  }

  Widget _contentWidget(BuildContext context) {
    if (backgroundImage.isEmptyString) {
      return Column(
        children: [onBuildHeadView(), Expanded(child: _buildRefreshWidget(context)), onBuildBottomView(context)],
      );
    } else {
      return Stack(
        children: [
          Container(
            decoration: BoxDecoration(
                color: backgroundColor,
                image: DecorationImage(
                  image: AssetImageProvider.render(backgroundImage, isProjectImage: !PluginUtils.instance.isRunPlugin),
                  alignment: backgroundAlignment,
                  fit: BoxFit.fitWidth,
                )),
          ),
          Column(
            children: [onBuildHeadView(), Expanded(child: _buildRefreshWidget(context)), onBuildBottomView(context)],
          )
        ],
      );
    }
  }

  /// 构建顶部导航条
  ActionBar? onActionBar() {
    return null;
  }

  /// 构建页面顶部视图
  Widget onBuildHeadView() {
    return Container();
  }

  /// 空视图对象
  Widget onEmptyRenderObject() {
    return Container();
  }

  /// 渲染视图top对象
  void onRenderTopObject(List<Widget> list) {}

  /// 构建渲染对象
  void onRenderObject(List<Widget> list) {}

  /// 渲染视图bottom对象
  Widget onBuildBottomView(BuildContext context) {
    return Container();
  }

  Widget _buildRefreshWidget(BuildContext context) {
    return RefreshWidget(
        listStyle: RefreshListStyle.custom,
        scrollMaxRatioLoad: 1,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: _bindListWidgets(),
        ),
        loadIndicatorWidget: Container(
          width: 86,
          alignment: Alignment.center,
          child: model.isNoMore
              ? Text(
                  RefreshUtils.instance.refreshConfig?.noMoreText() ?? "",
                  style: TextStyle(color: Color(0xb3333333), fontSize: 12),
                )
              : RefreshUtils.instance.refreshConfig?.loadingWidget(),
        ),
        enableRefresh: _isEnableRefresh,
        enableLoad: _isEnableLoad,
        initCall: (controller) {
          model.refreshController = controller;
        },
        onRefresh: (EasyRefreshController controller) async {
          model.refresh(context);
        },
        onLoad: (EasyRefreshController controller) {
          model.load(context);
        });
  }

  List<Widget> _bindListWidgets() {
    List<Widget> lst = [];
    onRenderTopObject(lst);
    if (model.isEmptyData) {
      lst.add(onEmptyRenderObject());
    } else {
      onRenderObject(lst);
    }
    return lst;
  }
}
